;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Definitions file                                              ;
;                                                               ;
;===============================================================;

#define VERSION_NUMBER "3.0.0"
#ifdef TI83PLUS
#define VERSION_COL 74
#endif
#ifdef TI84PLUSCE
#define VERSION_COL 128
#endif

#ifdef TI83PLUS
#include "ti83plus.inc"                         ; TI-83+ include file
#include "app.inc"                              ; SPASM app include
#include "get_key.inc"                          ; GET_KEY equates
#include "directin.inc"                         ; Direct Input equates
#endif

#ifdef TI84PLUSCE
#include "ti84pce.inc"                          ; TI-84+CE include file
#include "ab5dat.inc"
#include "ab5lvl.inc"
#include "get_key.inc"                          ; GET_KEY equates
#define bcall(xxxxxx) call xxxxxx
#define bcallz(xxxxxx) call z,xxxxxx
#define bcallnz(xxxxxx) call nz,xxxxxx
#define bcallc(xxxxxx) call c,xxxxxx
#define bcallnc(xxxxxx) call nc,xxxxxx
#define HUFF84PCE
#endif


#ifdef TI83PLUS
MREG                    = 2                     ; multibyte registers (DE, HL, etc.) use 2 bytes
_lcdbusy                = $4051
timer                   = $8448
apdMain                 = $8449
APD_MAIN                = $74
gbuf                    = plotsscreen
TILE_COLS               = 12
TILE_ROWS               = 8
ANIM_EXPLOSION          = 4
#endif

#ifdef TI84PLUSCE
MREG                    = 3                     ; multibyte registers (DE, HL, etc.) use 3 bytes
_lddehl                 = $021DA0

freemem                 = pixelshadow           ; 69090 bytes of available memory to use
startTiles              = freemem               ; 123 tiles x 256 bytes = 31488 bytes reserved
vars                    = startTiles+31488      ; 700 bytes reserved
appvarBuffer            = vars+700              ; 768 bytes reserved (compatibility with 83P)
chars                   = appvarBuffer+768      ; 64 chars x 11 bytes = 704 bytes reserved
sbuf                    = chars+704             ; 9600 bytes reserved used for title logo during menu, cut scene backgrounds, hud bar during gameplay, and epilogue background
palBackup               = sbuf+9600             ; 512 bytes reserved to be used by fadeOut & fadeIn to backup the palette
sprEnemy1               = palBackup+512         ; 4096 bytes reserved for 16 x unpacked sprites for enemy1
sprEnemy2               = sprEnemy1+4096        ; 4096 bytes reserved for 16 x unpacked sprites for enemy2
sprBoss                 = sprEnemy2+4096        ; 6144 bytes reserved for 24 x unpacked sprites for boss
codesBuffer             = sprBoss+6144          ; 321 bytes reserved for Ab5codes buffer
logoBuffer              = codesBuffer+321       ; 130 bytes reserved for episode logos during menu

huffTree                = cursorImage           ; 1024 bytes reserved for HuffExtr

pVram                   = mpLcdBase
pVbuf                   = cmdShadow             ; 3
pVbufWin                = pVbuf+3               ; 3
pDatFile                = pVbufWin+3            ; 3
pLvlFile                = pDatFile+3            ; 3
bossType                = pLvlFile+3            ; 1
fpsC                    = bossType+1            ; 3
fpsDisp                 = fpsC+3                ; 3
pcPage                  = fpsDisp+3             ; 1
pcNum                   = pcPage+1              ; 1
pcPageCount             = pcNum+1               ; 1
pcBuffer                = pcPageCount+1         ; 11

TILE_COLS               = 16
TILE_ROWS               = 12
VBUF_WIN_OFFSET         = ((240-(TILE_ROWS*16))/2*320)+((320-(TILE_COLS*16))/2)
ANIM_EXPLOSION          = 8
INI_PAL_SHIFT           = 32
FADE_DELAY              = 2000

vbuf1                   = vram
vbuf2                   = vram+(320*240)
textColour              = mpLcdPalette+(255*2)

INFO_BOX_X              = 40
INFO_BOX_Y              = 60
INFO_BOX_WIDTH          = 240
INFO_BOX_HEIGHT         = 120
INFO_BOX_TEXT_X         = (INFO_BOX_X+6)/2
INFO_BOX_TEXT_Y         = INFO_BOX_Y+19
#define INFO_LINE(yline,xpos,text,terminator) .db (yline*15)+INFO_BOX_TEXT_Y,(xpos*4)+INFO_BOX_TEXT_X,text,terminator

#define RECT_POS(rectX,rectY) ld de,rectY*256+(rectX/2)
#define RECT_SIZE(rectWidth,rectHeight) ld bc,rectWidth/2*256+rectHeight
#define RECT_HEIGHT(rectHeight) ld c,rectHeight
#define RECT_WIDTH_A() ld b,a \ srl b
#endif


SCROLL_DOWN             = 28
SCROLL_DOWN_BOSS        = (TILE_ROWS*8)-16
SCROLL_V_ADD            = (TILE_ROWS*8)-61
SCROLL_LEFT             = (TILE_COLS*8)-49
SCROLL_RIGHT            = 42

MISSION_TEXT            = 230
LEVEL_HEADER            = 24
LEVEL_AREA              = 60*48
LEVEL_EVENTS            = 783
LEVEL_DATA_SIZE         = MISSION_TEXT+LEVEL_HEADER+LEVEL_AREA+LEVEL_EVENTS
NUM_TILES               = 123
#ifdef TI83PLUS
TEMP_DATA_SIZE          = (NUM_TILES*8)+LEVEL_DATA_SIZE
#endif
#ifdef TI84PLUSCE
TEMP_DATA_SIZE          = LEVEL_DATA_SIZE
#endif

NUM_BULLETS             = 12                    ; max number of player bullets (only half of which can be used per player)
BULLET_SIZE             = 8                     ; size of each bulletTable entry
NUM_ANIMS               = 8                     ; max number of animations
ANIM_SIZE               = 6                     ; size of each animTable entry
NUM_ENEMIES             = 8                     ; max number of enemies
ENEMY_SIZE              = 10                    ; size of each enemyTable entry
NUM_AHOLES              = 3                     ; max number of alien holes
AHOLE_SIZE              = 8                     ; size of each aholeTable entry
EBULLET_SIZE            = 6
NUM_OBJECTS             = 8
OBJECT_SIZE             = 5

EBULLET_DAMAGE          = 8                     ; damage done by enemy bullets

#ifdef TI83PLUS
NUM_STARS               = 48
STAR_MAX_COORD          = 40
#define STAR_MUL add hl,hl \ add hl,hl \ add hl,hl \ ld d,h \ ld e,l \ add hl,hl \ add hl,de
#endif
#ifdef TI84PLUSCE
NUM_STARS               = 64
STAR_MAX_COORD          = 60
#define STAR_MUL add hl,hl \ add hl,hl \ add hl,hl \ add hl,hl \ add hl,hl \ add hl,hl
#endif
STAR_CLIP               = STAR_MAX_COORD * 2

;------------------------------------------------
; TI-84+CE colour defines
;------------------------------------------------
#ifdef TI84PLUSCE
COLOUR_BLACK            = 1
COLOUR_WHITE            = 31
COLOUR_DARKRED          = 35
COLOUR_DARKORANGE       = 43
COLOUR_LIGHTBLUE        = 60
#endif

;------------------------------------------------
; Operation defines
;------------------------------------------------
OP_SCF                                  = $37
OP_LD_A_N                               = $3E
OP_OR_A                                 = $B7

;------------------------------------------------
; tile defines
;------------------------------------------------
#include "tiledefs.asm"

TILE_RAMP               = RAMP_U                ; first ramp tile
TILE_FLOOR              = SSN_FLOOR_1           ; start of normal floor tiles
TILE_CRATER             = DIRT_CRATER           ; dirt crater tile
TILE_OUTERSPACE         = OUTER_SPACE           ; outer space tile
TILE_AIRDUCT            = AIR_DUCT_TL           ; first air duct tile
TILE_GARBAGECHUTE       = SSN_GARBAGE_CHUTE     ; garbage chute tile
TILE_COMPACTORL         = SSN_CRUSHER_L         ; compactor left tile
TILE_COMPACTORR         = SSN_CRUSHER_R         ; compactor right tile
TILE_VENTFAN            = SSN_VENT_FAN          ; vent fan tile
TILE_BOMB               = SSN_EXTRA_LIFE_TRAP   ; bomb tile
TILE_UNOPENALIENHOLE    = SSN_ALIEN_HOLE_HIDDEN ; unopen alien hole tile
TILE_OPENALIENHOLE      = SSN_ALIEN_HOLE_OPEN   ; open alien hole tile
TILE_ENEMYTRIGGER       = SSN_ENEMY_1_TRIGGER   ; first enemy trigger tile
TILE_FIREDOORV          = OPEN_FIRE_DOOR_V      ; vertical open fire door tile
TILE_FIREDOORH          = OPEN_FIRE_DOOR_H      ; horizontal open fire door tile
TILE_PUSHUP             = SSN_PUSH_U            ; push up tile
TILE_PUSHDOWN           = SSN_PUSH_D            ; push down tile
TILE_ENERGYSHIELD       = SSN_ENERGY_SHIELD_U   ; start of energy shield tiles
TILE_SOLID              = WALL_S                ; start of solid tiles
TILE_FIRSTWALL          = TILE_SOLID            ; first wall tile
TILE_COMPLEXWALL        = TILE_FIRSTWALL+11     ; start of the "complex" wall tiles that need to be double checked
TILE_LASTWALL           = TILE_FIRSTWALL+20     ; last wall tile
TILE_TRIGGEREDBOMB      = TRAP_TRIGGERED        ; triggered bomb tile
TILE_INTEXUL            = SSN_INTEX_CONSOLE_UL  ; facing up intex console left tile
TILE_INTEXUR            = SSN_INTEX_CONSOLE_UR  ; facing up intex console right tile
TILE_INTEXBL            = SSN_INTEX_CONSOLE_DL  ; facing down intex console left tile
TILE_INTEXBR            = SSN_INTEX_CONSOLE_DR  ; facing down intex console right tile
TILE_POWERDOME          = POWER_DOME_TL         ; start of working power dome tiles
TILE_DOOR               = SSN_CLOSED_DOOR_V     ; start of normal door tiles

OPEN_DOOR               = TILE_DOOR-SSN_OPEN_DOOR_V             ; difference between closed & open normal door tiles
PICKUPS                 = SSN_KEY_PASS          ; start of pickup tiles
CLOSE_FIRE_DOOR         = CLOSED_FIRE_DOOR_V-OPEN_FIRE_DOOR_V   ; difference between closed & open fire door tiles
ANIM_DOOR               = SSN_OPEN_DOOR_V-1     ; start of open door tiles minus 1 (to calc animation type)

;------------------------------------------------
; trigger/command defines
;------------------------------------------------
TRG_WALKEDONAREA        = $00
TRG_BOSSKILLED          = $01
TRG_COUNTDOWNSTARTED    = $02
TRG_FDOORCLOSED         = $03
TRG_PDOMEDESTROYED      = $04
TRG_HITWALL             = $05
TRG_COUNTERFINISHED     = $06
TRG_LEVELSTARTED        = $07
TRG_LIFEBOMB            = $08
TRG_FRAME4              = $09
TRG_INTEX               = $0A

CMD_CLEAREVENT          = $00
CMD_UPDATETILE          = $01
CMD_INIBOSS             = $02
CMD_STARTCOUNTDOWN      = $03
CMD_INCCOUNTER          = $04
CMD_SHOWTEXT            = $05
CMD_DARKON              = $06
CMD_DARKOFF             = $07
CMD_OBJCOMPLETE         = $08
CMD_NEWEVENT            = $09

;------------------------------------------------
; other defines
;------------------------------------------------
OPTION_DIFFICULTY       = %00000001
OPTION_PLAYERS          = %00000010
OPTION_RETREAT          = %00000100
OPTION_WALLS            = %00001000

SCORE_PICKUP            = 5                     ; score for collecting a pickup item
SCORE_ENEMY             = 20                    ; score for killing an enemy
SCORE_BOSS              = 250                   ; score for killing a boss
SCORE_LEVEL             = 200                   ; score for finishing a level

LFLAG_PASSCODE          = 0
LFLAG_INTEX             = 1
LFLAG_POWERDOMES        = 2
LFLAG_DARKLEVEL         = 3
;LFLAG_BIT4              = 4
;LFLAG_BIT5              = 5
LFLAG_CANFINISH         = 6
LFLAG_COUNTDOWNON       = 7

HITS_DOOR               = 48
HITS_POWERDOME          = 8
HITS_INTEX              = 4

CR_TWINLASER            = 1000
CR_FLAMEARC             = 2400
CR_PLASMARIFLE          = 3500
CR_FLAMETHROWER         = 4800
CR_KEYPACK              = 500
CR_AMMOCLIP             = 200
CR_FIRSTAID             = 500
CR_RADARSCANNER         = 1000
CR_EXTRALIFE            = 3000

MAX_HEALTH              = 127
INI_INVINCIBLE          = 80
INI_DEAD                = 48
INI_OPENALIENHOLE       = 100
INI_UNOPENALIENHOLE     = -50
MINE_DAMAGE             = MAX_HEALTH/4+1
ENEMY_CLOAKED           = 8

#ifdef TI83PLUS
PX                      = 0
PY                      = 2
PD                      = 4
PA                      = 5
PH                      = 7
PL                      = 8
PW                      = 9
PI                      = 10
PDD                     = 11
PHT                     = 12
PP                      = 13
PSY                     = 14
PSX                     = 15
PSH                     = 16
PK                      = 17
PS                      = 18
PCT                     = 20
#endif
#ifdef TI84PLUSCE
PX                      = 0
PY                      = 3
PD                      = 6
PA                      = 7
PH                      = 10
PL                      = 11
PW                      = 12
PI                      = 13
PDD                     = 14
PHT                     = 15
PP                      = 16
PSY                     = 17
PSX                     = 18
PSH                     = 19
PK                      = 20
PS                      = 21
PCT                     = 24
#endif

P1HMETER                = gbuf+(50*12)
P2HMETER                = gbuf+(50*12)+11
AMETER                  = gbuf+(50*12)+1

PONG_PADY               = 20
PONG_LEFT               = 6
PONG_RIGHT              = 96-8
PONG_TOP                = 3
PONG_BOTTOM             = 51
PMODE_SERVE             = 0
PMODE_RALLY             = 1
PMODE_ENDPOINT          = 2
PONG_FAST               = 3
PONG_FOLLOWMIN          = 30
PONG_FOLLOWMAX          = 70
PONG_WIN                = 15

TIP_WALKPAUSE           = 0
TIP_DOORS               = 1
TIP_KEYS                = 2
TIP_SHOOT               = 3
TIP_INTEX               = 4
TIP_CREDITS             = 5
TIP_WEAPONS             = 6
TIP_FIREDOORS           = 7

TIP_SHOOT_OFFSET        = 1425
TIP_INTEX_OFFSET        = 1418

; general pg1 achievements
ACH_HOOKCROOK_ID        = 0
ACH_HOOKCROOK_BIT       = %00000001
ACH_HOOKCROOK_OFFSET    = 0
ACH_PLAYPONG_ID         = 1
ACH_PLAYPONG_BIT        = %00000010
ACH_PLAYPONG_OFFSET     = 0
ACH_WINPONG_ID          = 2
ACH_WINPONG_BIT         = %00000100
ACH_WINPONG_OFFSET      = 0
ACH_COOPGAME_ID         = 3
ACH_COOPGAME_BIT        = %00001000
ACH_COOPGAME_OFFSET     = 0
ACH_SHOTDOOR_ID         = 4
ACH_SHOTDOOR_BIT        = %00010000
ACH_SHOTDOOR_OFFSET     = 0
; general pg2 achievements
ACH_ALLTOOLS_ID         = 5
ACH_ALLTOOLS_BIT        = %00000001
ACH_ALLTOOLS_OFFSET     = 1
ACH_0SECS_ID            = 6
ACH_0SECS_BIT           = %00000010
ACH_0SECS_OFFSET        = 1
ACH_GOODCHEAT_ID        = 7
ACH_GOODCHEAT_BIT       = %00000100
ACH_GOODCHEAT_OFFSET    = 1
ACH_BADCHEAT_ID         = 8
ACH_BADCHEAT_BIT        = %00001000
ACH_BADCHEAT_OFFSET     = 1
ACH_PONGCORE_ID         = 9
ACH_PONGCORE_BIT        = %00010000
ACH_PONGCORE_OFFSET     = 1
; episode I achievements
ACH_EP1_ID              = 10
ACH_EP1_BIT             = %00000001
ACH_EP1_OFFSET          = 2
ACH_EP1LP_ID            = 11
ACH_EP1LP_BIT           = %00000010
ACH_EP1LP_OFFSET        = 2
ACH_EP1L7TIME_ID        = 12
ACH_EP1L7TIME_BIT       = %00000100
ACH_EP1L7TIME_OFFSET    = 2
ACH_EP1BOSSCORE_ID      = 13
ACH_EP1BOSSCORE_BIT     = %00001000
ACH_EP1BOSSCORE_OFFSET  = 2
ACH_EP1PEACE_ID         = 14
ACH_EP1PEACE_BIT        = %00010000
ACH_EP1PEACE_OFFSET     = 2
; episode II achievements
ACH_EP2_ID              = 15
ACH_EP2_BIT             = %00000001
ACH_EP2_OFFSET          = 3
ACH_EP2LP_ID            = 16
ACH_EP2LP_BIT           = %00000010
ACH_EP2LP_OFFSET        = 3
ACH_EP2L7TIME_ID        = 17
ACH_EP2L7TIME_BIT       = %00000100
ACH_EP2L7TIME_OFFSET    = 3
ACH_EP2L4PUPS_ID        = 18
ACH_EP2L4PUPS_BIT       = %00001000
ACH_EP2L4PUPS_OFFSET    = 3
ACH_EP2L5DOORS_ID       = 19
ACH_EP2L5DOORS_BIT      = %00010000
ACH_EP2L5DOORS_OFFSET   = 3
; episode III achievements
ACH_EP3_ID              = 20
ACH_EP3_BIT             = %00000001
ACH_EP3_OFFSET          = 4
ACH_EP3LP_ID            = 21
ACH_EP3LP_BIT           = %00000010
ACH_EP3LP_OFFSET        = 4
ACH_EP3TMA5TIME_ID      = 22
ACH_EP3TMA5TIME_BIT     = %00000100
ACH_EP3TMA5TIME_OFFSET  = 4
ACH_EP3EASTER_ID        = 23
ACH_EP3EASTER_BIT       = %00001000
ACH_EP3EASTER_OFFSET    = 4
ACH_EP3PATHS_ID         = 24
ACH_EP3PATHS_BIT        = %00010000
ACH_EP3PATHS_OFFSET     = 4
; kills pg1 achievements
ACH_ENEMY1_ID           = 25
ACH_ENEMY1_BIT          = %00000001
ACH_ENEMY1_OFFSET       = 5
ACH_ENEMY2_ID           = 26
ACH_ENEMY2_BIT          = %00000010
ACH_ENEMY2_OFFSET       = 5
ACH_ENEMY3_ID           = 27
ACH_ENEMY3_BIT          = %00000100
ACH_ENEMY3_OFFSET       = 5
ACH_ENEMY4_ID           = 28
ACH_ENEMY4_BIT          = %00001000
ACH_ENEMY4_OFFSET       = 5
ACH_BOSS1_ID            = 29
ACH_BOSS1_BIT           = %00010000
ACH_BOSS1_OFFSET        = 5
; kills pg2 achievements
ACH_ENEMY5_ID           = 30
ACH_ENEMY5_BIT          = %00000001
ACH_ENEMY5_OFFSET       = 6
ACH_BOSS2_ID            = 31
ACH_BOSS2_BIT           = %00000010
ACH_BOSS2_OFFSET        = 6
ACH_BOSS3_ID            = 32
ACH_BOSS3_BIT           = %00000100
ACH_BOSS3_OFFSET        = 6
ACH_BOSSCORE1_ID        = 33
ACH_BOSSCORE1_BIT       = %00001000
ACH_BOSSCORE1_OFFSET    = 6
ACH_BOSSCORE2_ID        = 34
ACH_BOSSCORE2_BIT       = %00010000
ACH_BOSSCORE2_OFFSET    = 6
; items pg1 achievements
ACH_CREDIT1_ID          = 35
ACH_CREDIT1_BIT         = %00000001
ACH_CREDIT1_OFFSET      = 7
ACH_CREDIT2_ID          = 36
ACH_CREDIT2_BIT         = %00000010
ACH_CREDIT2_OFFSET      = 7
ACH_CREDIT3_ID          = 37
ACH_CREDIT3_BIT         = %00000100
ACH_CREDIT3_OFFSET      = 7
ACH_KEY1_ID             = 38
ACH_KEY1_BIT            = %00001000
ACH_KEY1_OFFSET         = 7
ACH_AMMO1_ID            = 39
ACH_AMMO1_BIT           = %00010000
ACH_AMMO1_OFFSET        = 7
; items pg2 achievements
ACH_CREDIT4_ID          = 40
ACH_CREDIT4_BIT         = %00000001
ACH_CREDIT4_OFFSET      = 8
ACH_FIRSTAID1_ID        = 41
ACH_FIRSTAID1_BIT       = %00000010
ACH_FIRSTAID1_OFFSET    = 8
ACH_EXTRALIVES1_ID      = 42
ACH_EXTRALIVES1_BIT     = %00000100
ACH_EXTRALIVES1_OFFSET  = 8
ACH_KEY2_ID             = 43
ACH_KEY2_BIT            = %00001000
ACH_KEY2_OFFSET         = 8
ACH_AMMO2_ID            = 44
ACH_AMMO2_BIT           = %00010000
ACH_AMMO2_OFFSET        = 8

ACH_HAS_VAR             = 25                    ; first achievement that has progress counter tracking

E1_NUMLEVELS            = 12
E1_LEVELOFFSET          = 0
E2_NUMLEVELS            = 12
E2_LEVELOFFSET          = E1_NUMLEVELS
E3_NUMLEVELS            = 30
E3_LEVELOFFSET          = E1_NUMLEVELS+E2_NUMLEVELS
NUMEPISODES             = 3
NUMLEVELS               = E1_NUMLEVELS+E2_NUMLEVELS+E3_NUMLEVELS

NUM_STARTS              = 3
NUM_FINISHES            = 3

NUM_PUPS_E2L4           = 103
NUM_DOORS_E2L5          = 44

FRAME_TIMER             = 3
INTO_GAME_DELAY         = 12

HSCORE_NAME_SIZE        = 16
HSCORE_DATA_SIZE        = 1+(HSCORE_NAME_SIZE*3)+6
HSCORE_OFFSET           = 1+(HSCORE_NAME_SIZE*3)
OPTIONS_SIZE            = 1
TIPS_DATA_SIZE          = 1
ACH_FLAGS_SIZE          = 9
ACH_COUNTER_SIZE        = 60
ACH_MISC_SIZE           = 1
ACH_DATA_SIZE           = ACH_FLAGS_SIZE+ACH_COUNTER_SIZE+ACH_MISC_SIZE
DATA_APPVAR_SIZE        = 1+HSCORE_DATA_SIZE+OPTIONS_SIZE+TIPS_DATA_SIZE+ACH_DATA_SIZE
DATA_APPVAR_VERSION     = 6
MAX_CODES               = 32
CODES_APPVAR_SIZE       = MAX_CODES*10+1
CODES_PER_PAGE          = 8
DATA_FILE_VERSION       = DATA_APPVAR_VERSION

#ifdef TI83PLUS
MEM_NEEDED              = TEMP_DATA_SIZE+13+DATA_APPVAR_SIZE+12     ; free memory required is size of an uncompressed level, plus VAT data of temp file, plus space for appvar in case it needs to be created
#endif
#ifdef TI84PLUSCE
MEM_NEEDED              = TEMP_DATA_SIZE+13+DATA_APPVAR_SIZE+12+CODES_APPVAR_SIZE+17
#endif


#ifdef TI83PLUS
systemBackup            = statVars              ; 256 bytes to backup system pointers
ram_huffExtr            = systemBackup+256      ; ram space to copy _ram_huffExtr & huffExtr routines (during page swapping)
vars                    = appBackupScreen       ; space for game variables (up to 700 bytes available, last 68 bytes reserved for gameflags)
videoBuffer             = saveSScreen           ; 768 bytes for gbuf copy
appvarBuffer            = saveSScreen
achBuffer               = appData               ; 256 bytes used for achievement routines to buffer strings & sprites
strBuffer               = op1                   ; space to copy strings to before using bcall(_vputs) (max 66 bytes including NULL terminator)
sprBuffer               = op1                   ; space to copy app page 0 sprites to before calling sprite routines on app page 1 (never used at the same time as strBuffer)
#endif


; e1CheatFlags
fastEnemyCheat          = 0
noWallsCheat            = 1
invincibleCheat         = 2
insaneModeCheat         = 3

; e2CheatFlags
weaponsCheat            = 0
noEnemiesCheat          = 1
noCountdownCheat        = 2
slowEnemyCheat          = 3
skipLevelCheat          = 4
invertedLcdCheat        = 5

; e3CheatFlags
keysCheat               = 0
ammoCheat               = 1
reverseCheat            = 2
moneyCheat              = 3
muntersCheat            = 4
boomersCheat            = 5

; scrollFlags
slTriggerCheck          = 0
srTriggerCheck          = 1
svTriggerCheck          = 2

; miscFlags
p1Blast                 = 0
p2Blast                 = 1
enemyPlane              = 2
passcodeUsed            = 3
loadingState            = 4
teleportCmd             = 5
appvarFlag              = 6
P1_BLAST                = %00000001
P2_BLAST                = %00000010

#define CPHLDE() push hl \ or a \ sbc hl,de \ pop hl        ; macro to replace bcall(_cphlde) for better speed
#define HLDIV8() srl h \ rr l \ srl h \ rr l \ srl h \ rr l ; macro to divide HL by 8 (16 bit only)
#define DEDIV8() srl d \ rr e \ srl d \ rr e \ srl d \ rr e ; macro to divide DE by 8 (16 bit only)
#define ATIMES8() add a,a \ add a,a \ add a,a               ; macro to multiply A by 8
#define ADIV8() ld c,0 \ srl a \ rr c \ srl a \ rr c \ srl a \ rr c \ rlc c \ rlc c \ rlc c ; macro to divide A by 8 and leave remainder in C

#ifdef TI83PLUS
#define MOVE9() rst $20                                     ; macro to move 9 bytes from (HL) to OP1
#define LDHLIND() ld a,(hl) \ inc hl \ ld h,(hl) \ ld l,a   ; macro to replace bcall(_ldhlind) for better speed
#define HTIMESL() ld b,h \ ld h,0 \ ld d,h \ ld e,h \ ex de,hl \ or a \ adc hl,de \ djnz $-2    ; macro to replace bcall(_htimesl) for better speed
#define HLTIMES8() add hl,hl \ add hl,hl \ add hl,hl        ; macro to multiply HL by 8
#define LDHLA() ld l,a \ ld h,0
#endif

#ifdef TI84PLUSCE
#define MOVE9() call _Mov9ToOP1
#define LDHLIND() ld hl,(hl)
#define HTIMESL() mlt hl
#define HLTIMES8() ld h,8 \ mlt hl
#define LDHLA() or a \ sbc hl,hl \ ld l,a
#define LDHL0() or a \ sbc hl,hl
#endif

;------------------------------------------------------------------------------
; Variable Name                  Location               Size    Description
;------------------------------------------------------------------------------
menuNum                         = vars                  ; 1     current menu number
difficulty                      = menuNum+1             ; 1     game difficulty
coop                            = difficulty+1          ; 1     co-op game flag
frame                           = coop+1                ; 1     frame counter
pauseFrame                      = frame+1               ; 1     frame counter used in pause screen
episode                         = pauseFrame+1          ; 1     current episode
level                           = episode+1             ; 1     current level
; >>>> these following variables are a set block (level header) >>>>
levelType                       = level+1               ; 1     level type (bits 0-3 type, bits 4-7 palette)
levelWidth                      = levelType+1           ; 1     level width (in blocks)
levelHeight                     = levelWidth+1          ; 1     level height (in blocks)
levelFlags                      = levelHeight+1         ; 1     level flags
enemy1                          = levelFlags+1          ; 1     enemy 1 data (bits 0-4 = type; bits 5-7 = level)
enemy2                          = enemy1+1              ; 1     enemy 2 data
aStart                          = enemy2+1              ; 9     array of start locations [prevLevel,x,y]
aFinish                         = aStart+9              ; 9     array of finish locations [x,y,nextLevel]
; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
startX                          = aFinish+9             ; 1     player x start
startY                          = startX+1              ; 1     player y start
counters                        = startY+1              ; 8     8 misc counters for use by events/scripting
time                            = counters+8            ; 1     countdown timer
prevLevel                       = time+1                ; 1     previous level that was finished
tileDataPtr                     = prevLevel+1           ; 2/3   pointer to start of uncompressed tile data
numEvents                       = tileDataPtr+MREG      ; 1     number of events
pEventTable                     = numEvents+1           ; 2/3   pointer to event table in level data
pEvents                         = pEventTable+MREG      ; 2/3   pointer to start of event scripts
state                           = pEvents+MREG          ; 1     game state: 0 = normal, 1 = boss
shake                           = state+1               ; 1     counter for screen shaking during the countdown sequence
screenX                         = shake+1               ; 2/3   absolute x coord of top left pixel on screen
screenY                         = screenX+MREG          ; 2/3   absolute y coord of top left pixel on screen
p1x                             = screenY+MREG          ; 2/3   player 1 x coord
p1y                             = p1x+MREG              ; 2/3   player 1 y coord
p1d                             = p1y+MREG              ; 1     player 1 direction
p1a                             = p1d+1                 ; 2/3   player 1 ammo
p1h                             = p1a+MREG              ; 1     player 1 health
p1l                             = p1h+1                 ; 1     player 1 lives
p1w                             = p1l+1                 ; 1     player 1 selected weapon
p1i                             = p1w+1                 ; 1     player 1 invincibility counter
p1dd                            = p1i+1                 ; 1     player 1 dead/death spin counter
p1ht                            = p1dd+1                ; 1     player 1 hurt flag
p1p                             = p1ht+1                ; 1     player 1 playing (active flag)
p1scry                          = p1p+1                 ; 1     player 1 y coord relative to screen
p1scrx                          = p1scry+1              ; 1     player 1 x coord relative to screen
p1sh                            = p1scrx+1              ; 1     player 1 show health meter counter
p1k                             = p1sh+1                ; 1     player 1 keys
p1s                             = p1k+1                 ; 2/3   player 1 score
p1ct                            = p1s+MREG              ; 1     player 1 collision tile
p2x                             = p1ct+1                ; 2/3   player 2 variables start here
p2y                             = p2x+MREG              ; 2/3
p2d                             = p2y+MREG              ; 1
p2a                             = p2d+1                 ; 2/3
p2h                             = p2a+MREG              ; 1
p2l                             = p2h+1                 ; 1
p2w                             = p2l+1                 ; 1
p2i                             = p2w+1                 ; 1
p2dd                            = p2i+1                 ; 1
p2ht                            = p2dd+1                ; 1
p2p                             = p2ht+1                ; 1
p2scry                          = p2p+1                 ; 1
p2scrx                          = p2scry+1              ; 1
p2sh                            = p2scrx+1              ; 1
p2k                             = p2sh+1                ; 1
p2s                             = p2k+1                 ; 2/3
p2ct                            = p2s+MREG              ; 1
pPlayer                         = p2ct+1                ; 2/3   pointer to active player data
player                          = pPlayer+MREG          ; 1     active player (0 or 1)
tempX                           = player+1              ; 2/3   temp storage
tempY                           = tempX+MREG            ; 2/3   temp storage
tempD                           = tempY+MREG            ; 1     temp storage
tempB                           = tempD+1               ; 1     temp storage
temp24                          = tempB+1               ; 3     temp 24-bit storage
scrollBits                      = temp24+3              ; 1     used in scrolling routines
scrollSkip                      = scrollBits+1          ; 2/3   used in scrolling routines
scrollMask                      = scrollSkip+MREG       ; 1     used in scrolling routines
scrollPtr                       = scrollMask+1          ; 2/3   used in scrolling routines
credits                         = scrollPtr+MREG        ; 2/3   credits
weaponTable                     = credits+MREG          ; 1     which weapons player has (bit flags)
numPickups                      = weaponTable+1         ; 1     # pickups collected on this level
numDoors                        = numPickups+1          ; 1     # doors opened on this level
showHLBuffer                    = numDoors+1            ; 9     string buffer for showHL & showAHL routines
bulletSlide                     = showHLBuffer+9        ; 1     slide value to use for flame arc and plasma rifle {0-2}
bulletTable                     = bulletSlide+1         ; 96    player bullets, 12 entries [type, direction, x{2}, y{2}, animation, slide]
animTable                       = bulletTable+96        ; 48    animations, 8 entries [counter, type, x{2}, y{2}]
enemyTable                      = animTable+48          ; 80    enemies, 8 entries [type, direction, x{2}, y{2}, counter, health, speed count, target]
ebullet                         = enemyTable+80         ; 8     enemy bullet, 1 entry [type, direction, x{2}, y{2}, animation, slide] (animation is unused, and slide always set to 1 to avoid sliding)
objectTable                     = ebullet+8             ; 40    objects, 8 entries [type, x{3}, y{3}]
collideObject1                  = objectTable+40        ; 6     collision detection object 1 [x{2}, width, y{2}, height]
collideObject2                  = collideObject1+6      ; 6     collision detection object 2 [x{2}, width, y{2}, height]
pCentreTable                    = collideObject2+6      ; 2/3   pointer to bullet centre table to use when moving a player or enemy bullet
fBulletCat                      = pCentreTable+MREG     ; 1     flag for either player bullet or enemy bullet (used when moving bullets)
passcodeVars                    = fBulletCat+1          ; 4     variables to be used in passcode, stored consecutively
passcode                        = passcodeVars+4        ; 11    passcode (10 chars plus NULL terminator)
aholeTable                      = passcode+11           ; 24    data for on-screen alien holes, 3 entries [counter, offset{3}, x{2}, y{2}]
overlayBuffer                   = aholeTable+24         ; 12    buffer for the dark level overlay (one row worth)
maskBuffer                      = overlayBuffer+12      ; 8     buffer for the dark level mask (one row worth)
contrastBuffer                  = maskBuffer+8          ; 1     place to store system contrast setting when fading in & out
treeaddr                        = contrastBuffer+1      ; 2/3   used by huffExtr
tempFile                        = treeaddr+MREG         ; 2/3   pointer to temp created program
pTiles                          = tempFile+MREG         ; 2     pointer to tile sprite storage (83P only)
pHscores                        = pTiles+MREG           ; 2/3   pointer to high score data in appvar in ram
pOptions                        = pHscores+MREG         ; 2/3   pointer to options flags
pTips                           = pOptions+MREG         ; 2/3   pointer to tips data in appvar in ram
pAchFlags                       = pTips+MREG            ; 2/3   pointer to achievement flags in appvar in ram
pAchVars                        = pAchFlags+MREG        ; 2/3   pointer to achievement variables in appvar in ram
menuItem                        = pAchVars+MREG         ; 1     selected menu item
wallTilePtr                     = menuItem+1            ; 2/3
wallFlags                       = wallTilePtr+MREG      ; 1
dly                             = wallFlags+1           ; 1
dlx                             = dly+1                 ; 1
sva                             = dlx+1                 ; 1
svgbuf                          = sva+1                 ; 2/3
svgbufEndRow                    = svgbuf+MREG           ; 2/3
svoffset                        = svgbufEndRow+MREG     ; 2/3
psb                             = svoffset+MREG         ; 1
enemyCt                         = psb+1                 ; 1
ascry                           = enemyCt+1             ; 1     average x coord relative to screen between active players
ascrx                           = ascry+1               ; 1     average y coord relative to screen between active players
vsc1                            = ascrx+1               ; 1
vsc2                            = vsc1+1                ; 1
enemyCollCheck                  = vsc2+1                ; 1     value to use in checkEnemyCollisions
putCMSRows2Put                  = enemyCollCheck+1      ; 1
putCMSClipMask                  = putCMSRows2Put+1      ; 1
putCMSBitMask                   = putCMSClipMask+1      ; 1
putCSClipMask                   = putCMSBitMask+1       ; 1
putCSRows2Put                   = putCSClipMask+1       ; 1
putCSBitMask                    = putCSRows2Put+1       ; 1
enemyXTest                      = putCSBitMask+1        ; 2/3
enemyYTest                      = enemyXTest+MREG       ; 2/3
bossSprCnt                      = enemyYTest+MREG       ; 1
bossSprPtr                      = bossSprCnt+1          ; 2/3
bossX                           = bossSprPtr+MREG       ; 2/3
bossY                           = bossX+MREG            ; 2/3
bossT                           = bossY+MREG            ; 1     boss target
bossScrX                        = bossT+1               ; 1
bossScrY                        = bossScrX+1            ; 1
bossState                       = bossScrY+1            ; 1
bossShot                        = bossState+1           ; 1
bossMoved                       = bossShot+1            ; 1
bossFlash                       = bossMoved+1           ; 1
cbecDamage                      = bossFlash+1           ; 1
bossHealth                      = cbecDamage+1          ; 2/3
cbbcDamage                      = bossHealth+MREG       ; 1
intexJumpTable                  = cbbcDamage+1          ; 3
intexOptions                    = intexJumpTable+MREG   ; 1
intexMenuItem                   = intexOptions+1        ; 1
intexCursorTable                = intexMenuItem+1       ; 2/3
ddmLeftOffset                   = intexCursorTable+MREG ; 1
ddmLineCounter                  = ddmLeftOffset+1       ; 1
ddmsShiftCounter                = ddmLineCounter+1      ; 1
randData                        = ddmsShiftCounter+1    ; 2/3
tileShotHits                    = randData+MREG         ; 1
tileShotPtr                     = tileShotHits+1        ; 2/3
intexShowCredits                = tileShotPtr+MREG      ; 1
palShift                        = intexShowCredits+1    ; 1     counter used for palette shifting on the 84+CE
pong1                           = palShift+1            ; 1     player 1 pong paddle position
pong2                           = pong1+1               ; 1     player 2 pong paddle position
pongBally                       = pong2+1               ; 1     pong ball y coord
pongBallx                       = pongBally+1           ; 1     pong ball x coord
pongMode                        = pongBallx+1           ; 1     mode of play in pong
pongMovey                       = pongMode+1            ; 1     y offset to use when moving ball
pongMovex                       = pongMovey+1           ; 1     x offset to use when moving ball
pongFrame                       = pongMovex+1           ; 1     frame counter
pong1score                      = pongFrame+1           ; 1     player 1 score
pong2score                      = pong1score+1          ; 1     player 2 score
pongRallyHits                   = pong2score+1          ; 1     # player hits in current rally
pongCalcFollow                  = pongRallyHits+1       ; 1     ball x coord to use to determine calc paddle speed
epScrPtr                        = pongCalcFollow+1      ; 2/3   pointer used in epilogue scrolling
epScrCnt                        = epScrPtr+MREG         ; 2/3   counter used in epilogue scrolling
aflagHookCrook                  = epScrCnt+MREG         ; 1     flag used for achievement "BY HOOK OR BY CROOK"
aflagLongPlay                   = aflagHookCrook+1      ; 1     flag used for episode long play achievements
aflagBossCore                   = aflagLongPlay+1       ; 1     flag used for achievement "OFFICER RIPLEY"
aflagAliens                     = aflagBossCore+1       ; 1     flag used for achievement "THE PACIFIST"
e1CheatFlags                    = aflagAliens+1         ; 1     episode 1 cheat flags
e2CheatFlags                    = e1CheatFlags+1        ; 1     episode 2 cheat flags
e3CheatFlags                    = e2CheatFlags+1        ; 1     episode 3 cheat flags
scrollFlags                     = e3CheatFlags+1        ; 1     scroll flags
miscFlags                       = scrollFlags+1         ; 1     misc game flags
pTeleport                       = miscFlags+1           ; 2/3   pointer to teleport coordinates
routeFlags                      = pTeleport+MREG        ; 1     route flags for Ep3

VARS_SIZE                       = miscFlags+1-vars
VARS_MEM_NEEDED                 = VARS_SIZE+2+14+2      ; variable space + 2 size bytes + vat entry + checksum


;------------------------------------------------
; variables used for star field and getting a string in menu - these double over existing variable allocations that only get used in-game
;------------------------------------------------
starTable                       = bulletTable
framexy                         = collideObject1
framez                          = framexy+1
pString                         = aholeTable
charCount                       = miscFlags
#ifdef TI83PLUS
__starXOffset                   = framez+1
__starXMove                     = __starXOffset+1
__starYMove                     = __starXMove+1
__starZMove                     = __starYMove+1
#endif

.end
