;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; High score table routines                                     ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; checkHighScore - check if player got a high score
;   input:  IX => player data
;           HL => string to use based on player
;   output: A = 0 if no high score, otherwise high score
;------------------------------------------------
checkHighScore:
        ld      (pPlayer),hl                    ; save string pointer
        ld      hl,(pHscores)
        ld      de,HSCORE_OFFSET
        add     hl,de                           ; HL => high scores
        ld      e,(ix+PS)
        ld      d,(ix+PS+1)                     ; DE = player score
        ld      b,3                             ; B = max 3 high scores to check
checkHighScoreLoop:
        push    hl                              ; save ptr
        LDHLIND()                               ; HL = high score to check
#ifdef TI84PLUSCE
        call    _setHLUto0
#endif
        bcall(_cphlde)                          ; check if player score beat this high score
        pop     hl                              ; restore ptr
        jr      c,newHighScore
        inc     hl
        inc     hl                              ; HL => next high score to check
        djnz    checkHighScoreLoop
        xor     a                               ; no high score
        ret                                     ; no high scores were beaten, return to main menu
newHighScore:
        ld      a,4
        sub     b                               ; A = slot number for high score (1 being the top score, 2 the 2nd, 3 the 3rd)
        cp      3                               ; check if it's the bottom entry on the table
        push    af                              ; save slot number
        jr      z,afterMoveHighScores           ; if so, no rearranging required
        ld      hl,(pHscores)
        push    hl
        ld      de,5+16
        add     hl,de
        ex      de,hl                           ; DE => 2nd high score name
        pop     hl
        ld      bc,HSCORE_OFFSET+2
        add     hl,bc                           ; HL => 2nd high score
moveHighScores:
        ld      c,(hl)
        inc     hl
        ld      b,(hl)                          ; BC = high score
        inc     hl
        ld      (hl),c
        inc     hl
        ld      (hl),b                          ; saved to next slot down
        ld      h,d
        ld      l,e                             ; HL => this high score name
        ld      bc,16
        add     hl,bc                           ; HL => where to copy it to
        ex      de,hl
        ld      bc,10                           ; BC = 10 characters to copy
        ldir                                    ; copy name down one slot in table
        ld      hl,(pHscores)
        push    hl
        ld      de,6
        add     hl,de
        ex      de,hl                           ; DE => 1st high score name
        pop     hl
        ld      bc,HSCORE_OFFSET
        add     hl,bc                           ; HL => 1st high score
        dec     a                               ; did player score the top score?
        jr      z,moveHighScores                ; if so, loop again to move top name down to 2nd
afterMoveHighScores:
        ld      hl,(pHscores)
        push    hl
        ld      de,HSCORE_OFFSET-2
        add     hl,de
        ex      de,hl                           ; DE => first slot less 1
        pop     hl
        ld      bc,6-16
        add     hl,bc                           ; HL => first slot name less 1
        pop     bc                              ; B = slot number for new high score
getHighNameSlot:
        push    de
        ld      de,16
        add     hl,de                           ; HL => next slot name
        pop     de
        inc     de
        inc     de                              ; DE => next slot score
        djnz    getHighNameSlot

        ; HL => where to store new high score name; DE => where to store new high score value
        ld      (pString),hl
        ex      de,hl                           ; HL => where to store score
        ld      c,(ix+PS)
        ld      b,(ix+PS+1)
        ld      (hl),c                          ; save score LSB
        inc     hl
        ld      (hl),b                          ; save score MSB
        xor     a
        ld      (charCount),a
        ld      a,6
        ld      (menuNum),a
        call    continueMenu                    ; runs the menu code until a name is entered

        ld      a,1                             ; a high score was made
        ret

.end
